import os
import sys
import tempfile


def main():
    filename = sys.argv[1]
    with tempfile.NamedTemporaryFile(delete=False) as fh:
        for line in open(filename):
            line = line.rstrip()
            fh.write(line + "\n")
        os.rename(filename, filename + ".bak")
        os.rename(fh.name, filename)


if __name__ == "__main__":
    main()



import json
import os
import platform

beamline_name = ""

# The following code is borrowed from PyXRF. It supposed to determine beamline name
#   based on PyXRF configuration file '/etc/pyxrf/pyxrf.json'

try:
    beamline_name = ""

    # Attempt to find the configuration file first
    config_path = "/etc/pyxrf/pyxrf.json"
    if os.path.isfile(config_path):
        try:
            with open(config_path, "r") as beamline_pyxrf:
                beamline_config_pyxrf = json.load(beamline_pyxrf)
                beamline_name = beamline_config_pyxrf["beamline_name"]
        except Exception as ex:
            raise IOError(f"Error while opening configuration file {config_path!r}") from ex

    else:
        # Otherwise try to identify the beamline using host name
        hostname = platform.node()
        beamline_names = {
            "xf03id": "HXN",
            "xf05id": "SRX",
            "xf08bm": "TES",
            "xf04bm": "XFM",
        }

        for k, v in beamline_names.items():
            if hostname.startswith(k):
                beamline_name = v

    if not beamline_name:
        raise Exception("Beamline is not identified")

    if beamline_name == "HXN":
        from pyxrf.db_config.hxn_db_config import db
    # elif beamline_name == "SRX":
    #     from pyxrf.db_config.srx_db_config import db
    # elif beamline_name == "XFM":
    #     from pyxrf.db_config.xfm_db_config import db
    # elif beamline_name == "TES":
    #     from pyxrf.db_config.tes_db_config import db
    else:
        db = None
        db_analysis = None
        print(f"Beamline Database is not used in pyxrf: beamline {beamline_name!r} is not supported")

except Exception as ex:
    db = None
    print(f"Beamline Database is not used in pyxrf: {ex}")



import collections
import numpy as np
from databroker import DataBroker as db
import logging


logger = logging.getLogger(__name__)


def _eval_scan_args(scan_args):
    """Evaluate scan arguments, replacing OphydObjects with NamedObjects"""

    class NamedObject:
        def __init__(self, name):
            self.name = name

    def no_op():
        def no_op_inner(*args, name=None, **kwargs):
            if name is not None:
                return NamedObject(name)

        return no_op_inner

    return eval(scan_args, collections.defaultdict(no_op))


step_1d = (
    "InnerProductAbsScan",
    "HxnInnerAbsScan",
    "InnerProductDeltaScan",
    "HxnInnerDeltaScan",
    "AbsScan",
    "HxnAbsScan",
    "DeltaScan",
    "HxnDeltaScan",
)

step_2d = ("OuterProductAbsScan", "HxnOuterAbsScan")
fly_scans = ("FlyPlan1D", "FlyPlan2D")


def get_scan_info(header):
    start_doc = header["start"]
    scan_args = start_doc["scan_args"]
    scan_type = start_doc["scan_type"]
    motors = None
    range_ = None
    scan_args = start_doc.get("scan_args", {})
    pyramid = False

    if scan_type in fly_scans:
        logger.debug("Scan %s (%s) is a fly scan (%s)", start_doc["scan_id"], start_doc["uid"], scan_type)
        dimensions = start_doc["dimensions"]
        motors = start_doc["axes"]
        pyramid = start_doc["fly_type"] == "pyramid"
        try:
            range_ = start_doc["scan_range"]
        except KeyError:
            try:
                range_ = [(float(scan_args["scan_start"]), float(scan_args["scan_end"]))]
            except (KeyError, ValueError):
                pass
    elif scan_type in step_2d:
        logger.debug("Scan %s (%s) is an ND scan (%s)", start_doc["scan_id"], start_doc["uid"], scan_type)
        # 2D mesh scan
        scan_args = _eval_scan_args(scan_args["args"])
        motors = [arg.name for arg in scan_args[::5]]
        dimensions = scan_args[3::5]
        range0 = scan_args[1::5]
        range1 = scan_args[2::5]
        range_ = list(zip(range0, range1))
    elif scan_type in step_1d or "num" in start_doc:
        logger.debug("Scan %s (%s) is a 1D scan (%s)", start_doc["scan_id"], start_doc["uid"], scan_type)
        # 1D scans
        dimensions = [int(scan_args["num"])]
        try:
            motors = [_eval_scan_args(scan_args["motor"]).name]
        except Exception:
            motors = []
    else:
        msg = "Unrecognized scan type (uid={} {})".format(start_doc["uid"], scan_type)
        raise RuntimeError(msg)

    num = np.product(dimensions)

    return {
        "num": num,
        "dimensions": dimensions,
        "motors": motors,
        "range": range_,
        "scan_args": scan_args,
        "pyramid": pyramid,
    }


class Scan(object):
    def __init__(self, header):
        self.header = header
        self.start_doc = header["start"]
        self.descriptors = header["descriptors"]
        self.key = None
        for key, value in get_scan_info(self.header).items():
            logger.debug("Scan info %s=%s", key, value)
            setattr(self, key, value)

    @property
    def filestore_keys(self):
        for desc in self.descriptors:
            for key, info in desc["data_keys"].items():
                try:
                    external = info["external"]
                except KeyError:
                    continue

                try:
                    source, info = external.split(":", 1)
                except Exception:
                    pass
                else:
                    source = source.lower()
                    if source in ("filestore",):
                        yield key

    @property
    def scan_id(self):
        return self.start_doc["scan_id"]

    def __repr__(self):
        return "{}(scan_id={})".format(self.__class__.__name__, self.scan_id)

    def __iter__(self):
        if self.key:
            for event in db.fetch_events(self.header, fill=False):
                yield event["data"][self.key]



#!/usr/bin/env python
"""
Created on May 23, 2013, last modified on June 19, 2013
@author: Cheng Chang (cheng.chang.ece@gmail.com)
         Computer Science Group, Computational Science Center
         Brookhaven National Laboratory

This code is for Differential Phase Contrast (DPC) imaging based on Fourier-shift fitting
implementation.

Reference: Yan, H. et al. Quantitative x-ray phase imaging at the nanoscale by multilayer
           Laue lenses. Sci. Rep. 3, 1307; DOI:10.1038/srep01307 (2013).

Test data is available at:
https://docs.google.com/file/d/0B3v6W1bQwN_AdjZwWmE3WTNqVnc/edit?usp=sharing
"""
from __future__ import print_function
import os
import numpy as np
import matplotlib.pyplot as plt
import PIL

from scipy.misc import imsave
from scipy.optimize import minimize
import time
import cStringIO as StringIO
import load_timepix

rss_cache = {}

rss_iters = 0


def get_beta(xdata):
    length = len(xdata)
    try:
        beta = rss_cache[length]
    except Exception:
        # beta = 1j * (np.arange(length) + 1 - (np.floor(length / 2.0) + 1))
        beta = 1j * (np.arange(length) - np.floor(length / 2.0))
        rss_cache[length] = beta

    return beta


def rss(v, xdata, ydata, beta):
    """Function to be minimized in the Nelder Mead algorithm"""
    fitted_curve = xdata * v[0] * np.exp(v[1] * beta)
    return np.sum(np.abs(ydata - fitted_curve) ** 2)


def pil_load(fn):
    im = PIL.Image.open(fn)

    def toarray(im, dtype=np.uint8):
        x_str = im.tostring("raw", im.mode)
        return np.fromstring(x_str, dtype)

    assert im.mode.startswith("I;16")
    if im.mode.endswith("B"):
        x = toarray(im, ">u2")
    else:
        x = toarray(im, "<u2")

    x.shape = im.size[1], im.size[0]
    return x.astype("=u2")


def load_file(fn, roi=None, bad_pixels=[], zip_file=None):
    """
    Load an image file
    """
    if os.path.exists(fn):
        im = load_timepix.load(fn)

    elif zip_file is not None:
        raise NotImplementedError

        # loading from a zip file is just about as fast (when not running in
        # parallel)
        f = zip_file.open(fn)
        stream = StringIO.StringIO()
        stream.write(f.read())
        f.close()

        stream.seek(0)
        im = plt.imread(stream, format="tif")
    else:
        raise Exception("File not found: %s" % fn)

    if bad_pixels is not None:
        for x, y in bad_pixels:
            im[x, y] = 0

    if roi is not None:
        x1, y1, x2, y2 = roi
        im = im[x1 : x2 + 1, y1 : y2 + 1]

    xline = np.sum(im, axis=1)
    yline = np.sum(im, axis=0)

    fx = np.fft.fftshift(np.fft.ifft(xline))
    fy = np.fft.fftshift(np.fft.ifft(yline))
    return im, fx, fy


def xj_test(filename, i, j, roi=None, bad_pixels=[], **kwargs):
    try:
        im, fx, fy = load_file(filename, zip_file=zip_file, roi=roi, bad_pixels=bad_pixels)
    except Exception as ex:
        print("Failed to load file %s: %s" % (filename, ex))
        return 0.0, 0.0, 0.0

    wx, wy = im.shape
    gx = np.sum(im[: wx / 2, :]) - np.sum(im[wx / 2 :, :])
    gy = np.sum(im[:, : wy / 2]) - np.sum(im[:, wy / 2 :])
    return 0, gx, gy


def run_dpc(
    filename,
    i,
    j,
    ref_fx=None,
    ref_fy=None,
    start_point=[1, 0],
    pixel_size=55,
    focus_to_det=1.46,
    dx=0.1,
    dy=0.1,
    energy=19.5,
    zip_file=None,
    roi=None,
    bad_pixels=[],
    max_iters=1000,
    solver="Nelder-Mead",
    invers=False,
):
    """
    All units in micron

    pixel_size
    focus_to_det: focus to detector distance
    dx: scan step size x
    dy: scan step size y
    energy: in keV
    """
    try:
        img, fx, fy = load_file(filename, zip_file=zip_file, roi=roi, bad_pixels=bad_pixels)
    except Exception as ex:
        print("Failed to load file %s: %s" % (filename, ex))
        return 0.0, 0.0, 0.0

    # vx = fmin(rss, start_point, args=(ref_fx, fx, get_beta(ref_fx)),
    #          maxiter=max_iters, maxfun=max_iters, disp=0)
    res = minimize(
        rss,
        start_point,
        args=(ref_fx, fx, get_beta(ref_fx)),
        method=solver,
        tol=1e-4,
        options=dict(maxiter=max_iters),
    )

    vx = res.x
    a = vx[0]
    if invers:
        gx = -vx[1]
    else:
        gx = vx[1]

    # vy = fmin(rss, start_point, args=(ref_fy, fy, get_beta(ref_fy)),
    #          maxiter=max_iters, maxfun=max_iters, disp=0)
    res = minimize(
        rss,
        start_point,
        args=(ref_fy, fy, get_beta(ref_fy)),
        method=solver,
        tol=1e-6,
        options=dict(maxiter=max_iters),
    )

    vy = res.x
    gy = vy[1]

    # print(i, j, vx[0], vx[1], vy[1])
    return a, gx, gy


def recon(gx, gy, dx=0.1, dy=0.1, pad=1, w=1.0):
    """
    Reconstruct the final phase image
    Parameters
    ----------
    gx : 2-D numpy array
        phase gradient along x direction

    gy : 2-D numpy array
        phase gradient along y direction

    dx : float
        scanning step size in x direction (in micro-meter)

    dy : float
        scanning step size in y direction (in micro-meter)

    pad : float
        padding parameter
        default value, pad = 1 --> no padding
                    p p p
        pad = 3 --> p v p
                    p p p

    w : float
        weighting parameter for the phase gradient along x and y direction when
        constructing the final phase image

    Returns
    ----------
    phi : 2-D numpy array
        final phase image

    References
    ----------
    [1] Yan, Hanfei, Yong S. Chu, Jorg Maser, Evgeny Nazaretski, Jungdae Kim,
    Hyon Chol Kang, Jeffrey J. Lombardo, and Wilson KS Chiu, "Quantitative
    x-ray phase imaging at the nanoscale by multilayer Laue lenses," Scientific
    reports 3 (2013).

    """

    rows, cols = gx.shape

    gx_padding = np.zeros((pad * rows, pad * cols), dtype="d")
    gy_padding = np.zeros((pad * rows, pad * cols), dtype="d")

    gx_padding[(pad // 2) * rows : (pad // 2 + 1) * rows, (pad // 2) * cols : (pad // 2 + 1) * cols] = gx
    gy_padding[(pad // 2) * rows : (pad // 2 + 1) * rows, (pad // 2) * cols : (pad // 2 + 1) * cols] = gy

    tx = np.fft.fftshift(np.fft.fft2(gx_padding))
    ty = np.fft.fftshift(np.fft.fft2(gy_padding))

    c = np.zeros((pad * rows, pad * cols), dtype=complex)

    mid_col = pad * cols // 2.0 + 1
    mid_row = pad * rows // 2.0 + 1

    ax = 2 * np.pi * (np.arange(pad * cols) + 1 - mid_col) / (pad * cols * dx)
    ay = 2 * np.pi * (np.arange(pad * rows) + 1 - mid_row) / (pad * rows * dy)

    kappax, kappay = np.meshgrid(ax, ay)

    c = -1j * (kappax * tx + w * kappay * ty)

    c = np.ma.masked_values(c, 0)
    c /= kappax ** 2 + w * kappay ** 2
    c = np.ma.filled(c, 0)

    c = np.fft.ifftshift(c)
    phi_padding = np.fft.ifft2(c)
    phi_padding = -phi_padding.real

    phi = phi_padding[(pad // 2) * rows : (pad // 2 + 1) * rows, (pad // 2) * cols : (pad // 2 + 1) * cols]

    return phi


def main(
    file_format="SOFC/SOFC_%05d.tif",
    dx=0.1,
    dy=0.1,
    ref_image=1,
    zip_file=None,
    rows=121,
    cols=121,
    start_point=[1, 0],
    pixel_size=55,
    focus_to_det=1.46e6,
    energy=19.5,
    pool=None,
    first_image=1,
    x1=None,
    x2=None,
    y1=None,
    y2=None,
    bad_pixels=[],
    solver="Nelder-Mead",
    display_fcn=None,
    invers=False,
):

    print("DPC")
    print("---")
    print("\tFile format: %s" % file_format)
    print("\tdx: %s" % dx)
    print("\tdy: %s" % dy)
    print("\trows: %s" % rows)
    print("\tcols: %s" % cols)
    print("\tstart point: %s" % start_point)
    print("\tpixel size: %s" % pixel_size)
    print("\tfocus to det: %s" % (focus_to_det / 1e6))
    print("\tenergy: %s" % energy)
    print("\tfirst image: %s" % first_image)
    print("\treference image: %s" % ref_image)
    print("\tsolver: %s" % solver)
    print("\tROI: (%s, %s)-(%s, %s)" % (x1, y1, x2, y2))

    t0 = time.time()

    roi = None
    if x1 is not None and x2 is not None:
        if y1 is not None and y2 is not None:
            roi = (x1, y1, x2, y2)

    # read the reference image: only one reference image
    reference, ref_fx, ref_fy = load_file(
        file_format % ref_image, zip_file=zip_file, roi=roi, bad_pixels=bad_pixels
    )

    a = np.zeros((rows, cols), dtype="d")
    gx = np.zeros((rows, cols), dtype="d")
    gy = np.zeros((rows, cols), dtype="d")

    dpc_settings = dict(
        start_point=start_point,
        pixel_size=pixel_size,
        focus_to_det=focus_to_det,
        dx=dx,
        dy=dy,
        energy=energy,
        zip_file=zip_file,
        ref_fx=ref_fx,
        ref_fy=ref_fy,
        roi=roi,
        bad_pixels=bad_pixels,
        solver=solver,
        invers=invers,
    )

    def get_filename(i, j):
        frame_num = first_image + i * cols + j

        # scan 1   9669
        #          12261 images
        #        = 21930
        # scan 2   21950
        # if frame_num >= 21930:
        #    frame_num += 20
        return file_format % frame_num

    # Wavelength in micron
    lambda_ = 12.4e-4 / energy
    if pool is None:
        for i in range(rows):
            trow = time.time()
            print("Row %d" % i, end="")
            rss_iters = 0
            for j in range(cols):
                _a, _gx, _gy = run_dpc(get_filename(i, j), i, j, **dpc_settings)
                a[i, j] = _a
                gx[i, j] = _gx
                gy[i, j] = _gy

            row_elapsed = 1000 * (time.time() - trow)
            print(" elapsed %.3fms" % row_elapsed, end=" ")
            print(" (per frame %.3fms, rss iters %d)" % (row_elapsed / cols, rss_iters))

    else:
        args = [(get_filename(i, j), i, j) for i in range(rows) for j in range(cols)]

        _t0 = time.time()
        try:
            if 1:
                fcn = run_dpc
            else:
                fcn = xj_test

            if display_fcn is not None:
                np.random.shuffle(args)

            results = [pool.apply_async(fcn, arg, kwds=dpc_settings) for arg in args]

            if display_fcn is not None:
                total_results = len(results)
                k = 0
                while k < total_results:
                    k = 0
                    for arg, result in zip(args, results):
                        if result.ready():
                            _a, _gx, _gy = result.get()
                            fn, i, j = arg

                            a[i, j] = _a
                            gx[i, j] = _gx
                            gy[i, j] = _gy
                            k += 1

                    try:
                        gx *= len(ref_fx) * pixel_size / (lambda_ * focus_to_det * 1e6)
                        gy *= len(ref_fy) * pixel_size / (lambda_ * focus_to_det * 1e6)
                        display_fcn(a, gx, gy, None)
                    except Exception as ex:
                        print("Failed to update display: (%s) %s" % (ex.__class__.__name__, ex))

                    time.sleep(1.0)

            pool.close()
            pool.join()
        except KeyboardInterrupt:
            print("Cancelled")
            return

        for arg, result in zip(args, results):
            fn, i, j = arg
            _a, _gx, _gy = result.get()
            a[i, j] = _a
            gx[i, j] = _gx
            gy[i, j] = _gy
            k += 1

        _t1 = time.time()
        elapsed = _t1 - _t0
        print(
            "Multiprocess elapsed=%.3f frames=%d (per frame %.3fms)"
            % (elapsed, rows * cols, 1000 * elapsed / (rows * cols))
        )

    gx *= len(ref_fx) * pixel_size / (lambda_ * focus_to_det * 1e6)
    gy *= len(ref_fy) * pixel_size / (lambda_ * focus_to_det * 1e6)

    dim = len(np.squeeze(gx).shape)

    if dim != 1:
        imsave("a.jpg", a)
        np.savetxt("a.txt", a)
        imsave("gx.jpg", gx)
        np.savetxt("gx.txt", gx)
        imsave("gy.jpg", gy)
        np.savetxt("gy.txt", gy)

        # -------------reconstruct the final phase image using gx and gy--------------------#
        phi = recon(gx, gy, dx, dy)
        imsave("phi.jpg", phi)
        np.savetxt("phi.txt", phi)

        t1 = time.time()
        print("Elapsed", t1 - t0)

        return a, gx, gy, phi

    else:
        """
        #plt.hold(False)
        plt.plot(np.squeeze(a), '-*')
        plt.savefig('a.jpg')
        np.savetxt('a.txt', a)

        plt.plot(np.squeeze(gx), '-*')
        plt.savefig('gx.jpg')
        np.savetxt('gx.txt', gx)

        plt.plot(np.squeeze(gy), '-*')
        plt.savefig('gy.jpg')
        np.savetxt('gy.txt', gy)

        t1 = time.time()
        print('Elapsed', t1 - t0)
        """

        phi = None
        return a, gx, gy, phi

    # plt.imshow(phi, cmap=cm.Greys_r)
    # plt.show()


if __name__ == "__main__":
    zip_file = None  # zipfile.ZipFile('SOFC.zip')
    main(zip_file=zip_file, processes=0, rows=121, cols=121)



"""
Created on Feb 23, 2017

@author: Mirna Lerotic, 2nd Look
"""
from __future__ import print_function, division
import sys
import os
import numpy as np
import multiprocessing as mp
import h5py
import PIL

try:
    from tifffile import imsave

    havetiff = True
except ImportError as ex:
    print("[!] Import error - tifffile not available. Tif files will not be saved")
    print("[!] (import error: {})".format(ex))
    havetiff = False


from db_config.db_config import db

# try:
#     from databroker import db, get_events
# except ImportError as ex:
#     print("[!] Unable to import DataBroker library.")

try:
    import hxntools
    import hxntools.handlers
    from hxntools.scan_info import ScanInfo
except ImportError as ex:
    print("[!] Unable to import hxntools library.")
    print("[!] (import error: {})".format(ex))
    hxntools = None


import dpc_kernel as dpc

version = "0.1.0"


def load_scan_from_mds(scan_id):
    hdrs = list(db(scan_id=scan_id))
    if len(hdrs) > 1:
        print(f"Multiple scans are available for scan_id {scan_id}. Processing the latest scan ...")
    hdr = hdrs[0]

    return ScanInfo(hdr)


def get_ref_from_mds(scan, first_image, file_store_key):
    if scan is None:
        return

    iter_ = iter(scan)

    first_image = max((1, first_image + 1))
    ref_image = None

    try:
        for i in range(first_image):
            ref_image = next(iter_)
    except StopIteration:
        print("Reference image #{} does not exist with data key {}" "".format(first_image, file_store_key))

    print(ref_image)

    return ref_image


def set_scan_from_scaninfo(scan):

    if scan.dimensions is None or len(scan.dimensions) == 0:
        return

    scan_range = scan.range
    print("Scan dimensions", scan.dimensions)
    print("Scan range:", scan_range)

    pyramid_scan = scan.pyramid

    if isinstance(scan_range, dict):
        scan_range = [scan_range[mtr] for mtr in scan.motors]

    if len(scan.dimensions) == 1:
        nx, ny = scan.dimensions[0], 1
        if scan_range is not None:
            dx = np.diff(scan_range[0]) / nx
            dy = 0.0
    else:
        nx, ny = scan.dimensions
        if scan_range is not None:
            dx = np.diff(scan_range[0]) / nx
            dy = np.diff(scan_range[1]) / ny

    cols = nx
    rows = ny

    return dx, dy, cols, rows, pyramid_scan


def load_data_hdf5(path):
    """
    Read images using the h5py lib
    """
    f = h5py.File(str(path), "r")
    entry = f["entry"]
    instrument = entry["instrument"]
    detector = instrument["detector"]
    dsdata = detector["data"]
    data = dsdata[...]

    return np.array(data)


def load_image_hdf5(path):

    data = load_data_hdf5(path)

    return data[0, :, :]


def save_results(
    a, gx, gy, phi, rx, ry, save_path, save_filename, scan_number, save_pngs=True, save_tif=True, save_txt=True
):

    save_filename = os.path.join(save_path, "S{0}_{1}".format(scan_number, save_filename))

    if os.path.isdir(save_path):
        if save_txt:
            a_path = save_filename + "_a.txt"
            np.savetxt(a_path, a)
            gx_path = save_filename + "_gx.txt"
            np.savetxt(gx_path, gx)
            gy_path = save_filename + "_gy.txt"
            np.savetxt(gy_path, gy)
            rx_path = save_filename + "_rx.txt"
            np.savetxt(rx_path, rx)
            ry_path = save_filename + "_ry.txt"
            np.savetxt(ry_path, ry)
            if phi is not None:
                phi_path = save_filename + "_phi.txt"
                np.savetxt(phi_path, phi)

        if save_pngs:
            a_path = save_filename + "_a.png"
            im = PIL.Image.fromarray((2.0 / a.ptp() * (a - a.min())).astype(np.uint8))
            im.save(a_path)
            gx_path = save_filename + "_gx.png"
            im = PIL.Image.fromarray((255.0 / gx.ptp() * (gx - gx.min())).astype(np.uint8))
            im.save(gx_path)
            gy_path = save_filename + "_gy.png"
            im = PIL.Image.fromarray((255.0 / gy.ptp() * (gy - gy.min())).astype(np.uint8))
            im.save(gy_path)
            rx_path = save_filename + "_rx.png"
            im = PIL.Image.fromarray((255.0 / rx.ptp() * (rx - rx.min())).astype(np.uint8))
            im.save(rx_path)
            ry_path = save_filename + "_ry.png"
            im = PIL.Image.fromarray((255.0 / ry.ptp() * (ry - ry.min())).astype(np.uint8))
            im.save(ry_path)
            if phi is not None:
                phi_path = save_filename + "_phi.png"
                im = PIL.Image.fromarray((255.0 / phi.ptp() * (phi - phi.min())).astype(np.uint8))
                im.save(phi_path)

        if save_tif and havetiff:
            if phi is not None:
                imgs = np.stack((a, gx, gy, rx, ry, phi))
                imsave(save_filename + ".tif", imgs.astype(np.float32))
            else:
                imgs = np.stack((a, gx, gy, rx, ry))
                imsave(save_filename + ".tif", imgs.astype(np.float32))

            # a_path = save_filename + '_a.tif'
            # imsave(a_path, a.astype(np.float32))
            # gx_path = save_filename + '_gx.tif'
            # imsave(gx_path, gx.astype(np.float32))
            # gy_path = save_filename + '_gy.tif'
            # imsave(gy_path, gy.astype(np.float32))
            # rx_path = save_filename + '_rx.tif'
            # imsave(rx_path, rx.astype(np.float32))
            # ry_path = save_filename + '_ry.tif'
            # imsave(ry_path, ry.astype(np.float32))
            # if phi is not None:
            #     phi_path = save_filename + '_phi.tif'
            #     imsave(phi_path, phi.astype(np.float32))

    else:
        print("Could not save results! Save directory {0} does not exist.".format(save_path))


# ----------------------------------------------------------------------
def init_scan_parameters():

    # Init settings
    scan_parameters = {
        "file_format": "S%d.h5",
        "dx": 0.1,
        "dy": 0.1,
        "ref_image": None,
        "rows": 121,
        "cols": 121,
        "start_point": [1, 0],
        "pixel_size": 55,
        "focus_to_det": 1.46,
        "energy": 19.5,
        "pool": None,
        "first_image": 0,
        "roi_x1": None,
        "roi_x2": None,
        "roi_y1": None,
        "roi_y2": None,
        "bad_pixels": [],
        "solver": "Nelder-Mead",
        "display_fcn": None,
        "random": 1,
        "pyramid": -1,
        "hang": 1,
        "swap": -1,
        "reverse_x": 1,
        "reverse_y": 1,
        "mosaic_x": 1,
        "mosaic_y": 1,
        "load_image": load_image_hdf5,
        "use_mds": False,
        "scan": None,
        "save_path": None,
        "pad": False,
    }

    return scan_parameters


# ----------------------------------------------------------------------
def read_scan_parameters_from_file(scan_parameters, param_filename):

    print("Reading scan parameters from ", param_filename)

    # try:
    if True:
        f = open(param_filename, "rt")
        for line in f:

            if line.startswith("#"):
                continue

            elif "step_size_dx_um" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["dx"] = float(slist[1])

            elif "step_size_dy_um" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["dy"] = float(slist[1])

            elif "cols_x" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["cols"] = int(slist[1])

            elif "rows_y" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["rows"] = int(slist[1])

            elif "pixel_size_um" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["pixel_size_um"] = float(slist[1])

            elif "detector_sample_distance" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["focus_to_det"] = float(slist[1])

            elif "energy_kev " in line.lower():
                slist = line.strip().split("=")
                scan_parameters["energy"] = float(slist[1])

            elif "roi_x1" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["roi_x1"] = int(slist[1])

            elif "roi_x2" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["roi_x2"] = int(slist[1])

            elif "roi_y1" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["roi_y1"] = int(slist[1])

            elif "roi_y2" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["roi_y2"] = int(slist[1])

            elif "mosaic_column_number_x" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["mosaic_x"] = int(slist[1])

            elif "mosaic_column_number_y" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["mosaic_y"] = int(slist[1])

            elif "solver" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["solver"] = slist[1].strip()

            elif "random" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["random"] = int(slist[1])

            elif "pyramid" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["pyramid"] = int(slist[1])

            elif "hang" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["hang"] = int(slist[1])

            elif "swap" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["swap"] = int(slist[1])

            elif "reverse_x" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["reverse_x"] = int(slist[1])

            elif "reverse_y" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["reverse_y"] = int(slist[1])

            elif "pad" in line.lower():
                slist = line.strip().split("=")
                scan_parameters["pad"] = int(slist[1])

        f.close()

        #     except:
        #         print('Could not read the script file. Exiting.')
        #         return

    # for key,value in scan_parameters.items():
    #     print(key + " = " + str(value))

    return scan_parameters


# ----------------------------------------------------------------------
def read_scan_parameters_from_datastore(scan_parameters):

    print("Reading scan parameters from DataStore")

    return scan_parameters


# ----------------------------------------------------------------------
def read_scan_parameters(scan_parameters, param_filename="", read_from_datastore=False):

    if read_from_datastore:
        scan_parameters = read_scan_parameters_from_datastore(scan_parameters)
    else:
        scan_parameters = read_scan_parameters_from_file(scan_parameters, param_filename)

    return scan_parameters


# ----------------------------------------------------------------------
def parse_scan_range(scan_range, scan_numbers, str_scan_range):

    slist = str_scan_range.split(",")
    for item in slist:
        if "-" in item:
            slist = item.split("-")
            scan_range.append((int(slist[0].strip()), int(slist[1].strip())))
        else:
            scan_numbers.append(int(item.strip()))

    return scan_range, scan_numbers


# ----------------------------------------------------------------------
def parse_script(script_file):

    scan_range = []
    scan_numbers = []
    every_nth_scan = 1
    get_data_from_datastore = 0
    scan_header_index = 0
    data_directory = ""
    read_params_from_datastore = 0
    parameter_file = ""
    processes = 1
    scan_header_index = 0
    file_format = "S{0}.h5"
    save_filename = "results"
    file_store_key = ""
    save_path = ""
    save_pngs = 1
    save_txt = 1

    try:
        # if True:
        f = open(script_file, "rt")
        for line in f:

            if line.startswith("#"):
                continue

            elif "scan_range" in line.lower():
                slist = line.strip().split("=")
                scan_range, scan_numbers = parse_scan_range(scan_range, scan_numbers, slist[1])

            elif "scan_numbers" in line.lower():
                slist = line.strip().split("=")
                slist = slist[1].split(",")
                for item in slist:
                    scan_numbers.append(int(item.strip()))

            elif "every_nth_scan" in line.lower():
                slist = line.strip().split("=")
                every_nth_scan = int(slist[1])

            elif "get_data_from_datastore" in line.lower():
                slist = line.strip().split("=")
                get_data_from_datastore = int(slist[1])

            elif "read_params_from_datastore" in line.lower():
                slist = line.strip().split("=")
                read_params_from_datastore = int(slist[1])

            elif "processes" in line.lower():
                slist = line.strip().split("=")
                processes = int(slist[1])

            elif "scan_header_index" in line.lower():
                slist = line.strip().split("=")
                scan_header_index = int(slist[1])

            elif "data_directory" in line.lower():
                slist = line.strip().split("=")
                data_directory = slist[1].strip()

            elif "save_path" in line.lower():
                slist = line.strip().split("=")
                save_path = slist[1].strip()

            elif "save_filename" in line.lower():
                slist = line.strip().split("=")
                save_filename = slist[1].strip()

            elif "save_pngs" in line.lower():
                slist = line.strip().split("=")
                save_pngs = int(slist[1])

            elif "save_txt" in line.lower():
                slist = line.strip().split("=")
                save_txt = int(slist[1])

            elif "parameter_file" in line.lower():
                slist = line.strip().split("=")
                parameter_file = slist[1].strip()

            elif "file_format" in line.lower():
                slist = line.strip().split("=")
                file_format = slist[1].strip()

            elif "file_store_key" in line.lower():
                slist = line.strip().split("=")
                file_store_key = slist[1].strip()

        f.close()

    except Exception:
        print("Could not read the script file. Exiting.")
        exit()

    print("Script setup:")
    print("scan_range", scan_range)
    print("scan_numbers", scan_numbers)
    print("scan_header_index", scan_header_index)
    print("file_store_key", file_store_key)
    print("every_nth_scan", every_nth_scan)
    print("get_data_from_datastore", get_data_from_datastore)
    print("data_directory", data_directory)
    print("file_format", file_format)
    print("read_params_from_datastore", read_params_from_datastore)
    print("parameter_file", parameter_file)
    print("processes", processes)
    print("save_path", save_path)
    print("save_filename", save_filename)
    print("save_pngs", save_pngs)
    print("save_txt", save_txt)

    return (
        scan_range,
        scan_numbers,
        every_nth_scan,
        get_data_from_datastore,
        data_directory,
        read_params_from_datastore,
        parameter_file,
        processes,
        scan_header_index,
        file_format,
        file_store_key,
        save_path,
        save_filename,
        save_pngs,
        save_txt,
    )


""" ------------------------------------------------------------------------------------------------"""


def run_batch(script_file):

    print("Parsing script ", script_file)
    (
        scan_range,
        scan_numbers,
        every_nth_scan,
        get_data_from_datastore,
        data_directory,
        read_params_from_datastore,
        parameter_file,
        processes,
        scan_header_index,
        file_format,
        file_store_key,
        save_path,
        save_filename,
        save_pngs,
        save_txt,
    ) = parse_script(script_file)

    if get_data_from_datastore == 1:
        if hxntools is None:
            print("Warning! Cannot read scan parameters from DataStore because hnxtools library is not available.")
        print("Reading data from DataStore.")
    else:
        print("Reading data from .h5 files.")

    calc_scan_numbers = np.array((), dtype=np.int)
    calc_scan_numbers = np.append(calc_scan_numbers, np.array(scan_numbers, dtype=np.int))
    # Get scan numbers from the range
    for item in scan_range:
        calc_scan_numbers = np.append(
            calc_scan_numbers, np.arange(item[0], item[1] + 1, every_nth_scan, dtype=np.int)
        )

    scan_parameters = init_scan_parameters()

    try:
        scan_parameters = read_scan_parameters(scan_parameters, param_filename=parameter_file)
    except Exception:
        print("Could not read scan parameters from parameter file {}. Using defaults.".format(parameter_file))

    dpc_settings = {
        "file_format": "",
        "save_path": data_directory,
        "dx": scan_parameters["dx"],
        "dy": scan_parameters["dy"],
        "x1": scan_parameters["roi_x1"],
        "y1": scan_parameters["roi_y1"],
        "x2": scan_parameters["roi_x2"],
        "y2": scan_parameters["roi_y2"],
        "pixel_size": scan_parameters["pixel_size"],
        "focus_to_det": scan_parameters["focus_to_det"],
        "energy": scan_parameters["energy"],
        "rows": scan_parameters["rows"],
        "cols": scan_parameters["cols"],
        "mosaic_y": scan_parameters["mosaic_y"],
        "mosaic_x": scan_parameters["mosaic_x"],
        "swap": scan_parameters["swap"],
        "reverse_x": scan_parameters["reverse_x"],
        "reverse_y": scan_parameters["reverse_y"],
        "random": scan_parameters["random"],
        "pyramid": scan_parameters["pyramid"],
        "pad": scan_parameters["pad"],
        "hang": scan_parameters["hang"],
        "ref_image": scan_parameters["ref_image"],
        "first_image": scan_parameters["first_image"],
        "solver": scan_parameters["solver"],
        "scan": None,
        "use_mds": scan_parameters["use_mds"],
        "calculate_results": True,
    }

    n_scans = calc_scan_numbers.size

    for i_scan in range(n_scans):

        scan_filename = os.path.join(data_directory, file_format.format(calc_scan_numbers[i_scan]))
        print("\nProcessing scan number ", calc_scan_numbers[i_scan])

        dpc_settings["file_format"] = scan_filename
        dpc_settings["ref_image"] = scan_filename
        dpc_settings["scan"] = calc_scan_numbers[i_scan]

        if get_data_from_datastore:
            load_image = dpc.load_image_filestore
            dpc_settings["file_format"] = ""
            dpc_settings["ref_image"] = ""
            dpc_settings["use_hdf5"] = False
            dpc_settings["use_mds"] = True

            try:
                scan_id = int(calc_scan_numbers[i_scan])
                mds_scan = load_scan_from_mds(scan_id)
            except Exception as ex:
                print(
                    "Filestore load failed (datum={}): ({}) {}"
                    "".format(calc_scan_numbers[i_scan], ex.__class__.__name__, ex)
                )
                raise
            mds_scan.key = file_store_key

            dpc_settings["scan"] = mds_scan

            dpc_settings["ref_image"] = get_ref_from_mds(mds_scan, scan_parameters["first_image"], file_store_key)

            if read_params_from_datastore == 1:
                dx, dy, cols, rows, pyramid_scan = set_scan_from_scaninfo(mds_scan)
                dpc_settings["dx"] = dx
                dpc_settings["dy"] = dy
                dpc_settings["rows"] = rows
                dpc_settings["cols"] = cols
                dpc_settings["pyramid"] = pyramid_scan

        else:
            print("\nProcessing scan ", scan_filename)
            load_image = load_image_hdf5
            dpc_settings["use_hdf5"] = True

        if processes == 0:
            print(
                "Error - number of processes in myscript.txt is equal to 0. "
                "Please set to minimum 1 with processes = 1."
            )
            exit()
        else:
            pool = mp.Pool(processes=processes)

        # Run the analysis
        a, gx, gy, phi, rx, ry = dpc.main(pool=pool, display_fcn=None, load_image=load_image, **dpc_settings)

        save_results(
            a,
            gx,
            gy,
            phi,
            rx,
            ry,
            save_path,
            save_filename,
            calc_scan_numbers[i_scan],
            save_pngs=save_pngs,
            save_tif=True,
            save_txt=save_txt,
        )

    print("DPC finished")


""" ------------------------------------------------------------------------------------------------"""


def main():

    try:
        script_file = sys.argv[1]
    except Exception:
        print("Error - Script file not given.\nUsage: python dpc_batch.py myscript.txt")
        exit()

    run_batch(script_file)


if __name__ == "__main__":
    main()



"""
Created on Feb 23, 2017

@author: Mirna Lerotic, 2nd Look
"""
from __future__ import division
import sys
import os

from PyQt5.QtWidgets import (
    QMainWindow,
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QGroupBox,
    QCheckBox,
    QLabel,
    QLineEdit,
    QPushButton,
    QFileDialog,
    QMessageBox,
    QApplication,
    QTextEdit,
)
from PyQt5.QtGui import QPalette, QColor, QIntValidator, QTextCursor
from PyQt5.QtCore import Qt, QCoreApplication, QSettings


import dpc_batch as dpc

version = "1.0.2"

# #----------------------------------------------------------------------
# class EmittingStream(QObject):
#
#     textWritten = pyqtSignal(str)
#
#     def write(self, text):
#         self.textWritten.emit(str(text))

""" ------------------------------------------------------------------------------------------------"""


class MainFrame(QMainWindow):
    def __init__(self):
        super(MainFrame, self).__init__()

        self.settings = QSettings("BNL", "DPC-BATCH-GUI")

        self.script_file = ""

        try:
            val = self.settings.value("scan_range").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.scan_range = val
        try:
            val = self.settings.value("scan_nums").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.scan_nums = val
        try:
            val = self.settings.value("every_n").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 1
        self.every_n = val
        try:
            val = self.settings.value("load_params_datastore").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 0
        self.read_data_from_datastore = val
        try:
            val = self.settings.value("filestore_key").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = "merlin1"
        self.filestore_key = val
        try:
            val = self.settings.value("data_dir").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.data_directory = val
        try:
            val = self.settings.value("file_format").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = "S{0}.h5"
        self.file_format = val
        try:
            val = self.settings.value("load_params_datastore").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 0
        self.load_params_from_broker = val
        try:
            val = self.settings.value("param_file").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.parameter_file = val
        try:
            val = self.settings.value("processes").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 1
        self.processes = val
        try:
            val = self.settings.value("save_dir").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.save_dir = val
        try:
            val = self.settings.value("save_fn").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = ""
        self.save_filename = val
        try:
            val = self.settings.value("save_png").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 0
        self.save_png = val
        try:
            val = self.settings.value("save_txt").toPyObject()
        except AttributeError:
            val = None
        if val is None:
            val = 0
        self.save_txt = val

        self.resize(600, 720)
        self.setWindowTitle("DPC Batch v.{}".format(version))

        pal = QPalette()
        self.setAutoFillBackground(True)
        pal.setColor(QPalette.Window, QColor("white"))
        self.setPalette(pal)

        self.mainWidget = QWidget(self)
        self.setCentralWidget(self.mainWidget)

        vbox = QVBoxLayout(self.mainWidget)
        vbox.setContentsMargins(20, 10, 20, 20)

        sizer1 = QGroupBox("Scans")
        vbox1 = QVBoxLayout()

        self.cb_usedatastore = QCheckBox("  Read the Data from DataStore", self)
        self.cb_usedatastore.setChecked(self.read_data_from_datastore)
        self.cb_usedatastore.stateChanged.connect(self.OnUseDataStore)
        vbox1.addWidget(self.cb_usedatastore)

        hbox = QHBoxLayout()
        l1 = QLabel("Scan numbers & ranges \t", self)
        self.tc_scan_range = QLineEdit(self)
        self.tc_scan_range.setAlignment(Qt.AlignLeft)
        self.tc_scan_range.setText(self.scan_range)
        l1.setToolTip("Set scan numbers and ranges. Example: 2, 3-5, 7-15, 23, 30-55")
        self.tc_scan_range.setToolTip("Set scan numbers and ranges. Example: 2, 3-5, 7-15, 23, 30-55")
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_scan_range)
        vbox1.addLayout(hbox)

        hbox = QHBoxLayout()
        l2 = QLabel("Process every n-th scan \t", self)
        self.ntc_every_n = QLineEdit(self)
        self.ntc_every_n.setValidator(QIntValidator(1, 99999, self))
        self.ntc_every_n.setAlignment(Qt.AlignRight)
        self.ntc_every_n.setText(str(self.every_n))
        hbox.addWidget(l2)
        hbox.addWidget(self.ntc_every_n)
        hbox.addStretch(1)
        vbox1.addLayout(hbox)

        # hbox = QHBoxLayout()
        # l1 = QLabel('Scan numbers \t', self)
        # self.tc_scans = QLineEdit(self)
        # self.tc_scans.setAlignment(Qt.AlignLeft)
        # self.tc_scans.setText(str(self.scan_nums))
        # l1.setToolTip('Set scan numbers. Example: 1, 24, 26')
        # self.tc_scans.setToolTip('Set scan numbers. Example: 1, 24, 26')
        # hbox.addWidget(l1)
        # hbox.addWidget(self.tc_scans)
        # vbox1.addLayout(hbox)

        hbox = QHBoxLayout()
        l1 = QLabel("Filestore key \t", self)
        self.tc_fskey = QLineEdit(self)
        self.tc_fskey.setAlignment(Qt.AlignLeft)
        self.tc_fskey.setText(self.filestore_key)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_fskey)
        vbox1.addLayout(hbox)

        hbox = QHBoxLayout()
        l1 = QLabel("Data Directory \t", self)
        self.tc_datadir = QLineEdit(self)
        self.tc_datadir.setAlignment(Qt.AlignLeft)
        l1.setToolTip("Data Direcory if not using datastore.")
        self.tc_datadir.setToolTip("Data Direcory if not using datastore.")
        self.tc_datadir.setText(self.data_directory)
        self.button_d1 = QPushButton("Browse")
        self.button_d1.clicked.connect(self.OnSelectDataDir)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_datadir)
        hbox.addWidget(self.button_d1)
        vbox1.addLayout(hbox)

        hbox = QHBoxLayout()
        l1 = QLabel("File format \t", self)
        self.tc_format = QLineEdit(self)
        self.tc_format.setAlignment(Qt.AlignLeft)
        self.tc_format.setToolTip("Data file format.")
        self.tc_format.setText(self.file_format)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_format)
        vbox1.addLayout(hbox)

        sizer1.setLayout(vbox1)
        vbox.addWidget(sizer1)

        sizer3 = QGroupBox("Scan Parameters")
        vbox3 = QVBoxLayout()

        self.cb_paramsdatastore = QCheckBox("  Read the Parameters from DataStore", self)
        self.cb_paramsdatastore.setChecked(self.load_params_from_broker)
        vbox3.addWidget(self.cb_paramsdatastore)

        hbox = QHBoxLayout()
        l1 = QLabel("Parameter file \t", self)
        self.tc_paramfile = QLineEdit(self)
        self.tc_paramfile.setAlignment(Qt.AlignLeft)
        self.tc_paramfile.setText(self.parameter_file)
        button_d3 = QPushButton("Select")
        button_d3.clicked.connect(self.OnSelectParamFile)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_paramfile)
        hbox.addWidget(button_d3)
        vbox3.addLayout(hbox)

        sizer3.setLayout(vbox3)
        vbox.addWidget(sizer3)

        sizer2 = QGroupBox("Run configurations")
        vbox2 = QVBoxLayout()
        hbox = QHBoxLayout()

        l2 = QLabel("Processes  \t", self)
        self.ntc_processes = QLineEdit(self)
        self.ntc_processes.setValidator(QIntValidator(1, 64, self))
        self.ntc_processes.setAlignment(Qt.AlignRight)
        self.ntc_processes.setText(str(self.processes))
        hbox.addWidget(l2)
        hbox.addWidget(self.ntc_processes)
        hbox.addStretch(1)
        vbox2.addLayout(hbox)

        sizer2.setLayout(vbox2)
        vbox.addWidget(sizer2)

        sizer4 = QGroupBox("Saving Results")
        vbox4 = QVBoxLayout()
        hbox = QHBoxLayout()

        hbox = QHBoxLayout()
        l1 = QLabel("Save Directory \t", self)
        self.tc_savedir = QLineEdit(self)
        self.tc_savedir.setAlignment(Qt.AlignLeft)
        l1.setToolTip("Data Direcory where results will be stored.")
        self.tc_savedir.setToolTip("Data Direcory where results will be stored.")
        self.tc_savedir.setText(self.save_dir)
        button_d2 = QPushButton("Browse")
        button_d2.clicked.connect(self.OnSelectSaveDir)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_savedir)
        hbox.addWidget(button_d2)
        vbox4.addLayout(hbox)

        hbox = QHBoxLayout()
        l1 = QLabel("Save Filename \t", self)
        self.tc_savefn = QLineEdit(self)
        self.tc_savefn.setAlignment(Qt.AlignLeft)
        self.tc_savefn.setText(self.save_filename)
        hbox.addWidget(l1)
        hbox.addWidget(self.tc_savefn)
        vbox4.addLayout(hbox)

        self.cb_savepng = QCheckBox("  Save results as .png files", self)
        self.cb_savepng.setChecked(self.save_png)
        vbox4.addWidget(self.cb_savepng)

        self.cb_savetxt = QCheckBox("  Save results as .txt files", self)
        self.cb_savetxt.setChecked(self.save_txt)
        vbox4.addWidget(self.cb_savetxt)

        cb_savetif = QCheckBox("  Save results as .tif files", self)
        cb_savetif.setChecked(True)
        cb_savetif.setDisabled(True)
        vbox4.addWidget(cb_savetif)

        sizer4.setLayout(vbox4)
        vbox.addWidget(sizer4)

        hbox = QHBoxLayout()
        self.button_save = QPushButton("Save")
        self.button_save.clicked.connect(self.OnSave)
        hbox.addWidget(self.button_save)
        self.button_start = QPushButton("Start")
        self.button_start.clicked.connect(self.OnStart)
        hbox.addWidget(self.button_start)
        vbox.addLayout(hbox)

        self.console_info = QTextEdit(self)
        self.console_info.setReadOnly(True)
        vbox.addWidget(self.console_info)

        # sys.stdout = EmittingStream(textWritten=self.ConsoleOutput)

        self.show()
        if sys.platform == "darwin":
            self.raise_()

        self.OnUseDataStore()

    # ----------------------------------------------------------------------
    def __del__(self):
        sys.stdout = sys.__stdout__

    # ----------------------------------------------------------------------
    def ConsoleOutput(self, text):

        cursor = self.console_info.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        self.console_info.setTextCursor(cursor)
        self.console_info.ensureCursorVisible()

    # ----------------------------------------------------------------------
    def OnBrowseDir(self):

        directory = QFileDialog.getExistingDirectory(
            self, "Choose a directory", "", QFileDialog.ShowDirsOnly | QFileDialog.ReadOnly
        )

        if directory == "":
            return ""

        return str(directory)

    # ----------------------------------------------------------------------
    def OnSelectDataDir(self):
        datapath = self.OnBrowseDir()
        self.tc_datadir.setText(str(os.path.abspath(datapath)))

    # ----------------------------------------------------------------------
    def OnSelectSaveDir(self):
        datapath = self.OnBrowseDir()
        self.tc_savedir.setText(str(os.path.abspath(datapath)))

    # ----------------------------------------------------------------------
    def OnSelectParamFile(self):
        paramfile = QFileDialog.getOpenFileName(self, "Choose a parameter file", "", "Text file (*.txt)")[0]
        self.tc_paramfile.setText(str(os.path.abspath(paramfile)))

    # ----------------------------------------------------------------------
    def OnUseDataStore(self):
        if self.cb_usedatastore.isChecked():
            self.read_data_from_datastore = 1
            self.tc_datadir.setDisabled(True)
            self.button_d1.setDisabled(True)
            self.tc_format.setDisabled(True)
            self.ntc_every_n.setDisabled(False)
            self.tc_fskey.setDisabled(False)
        else:
            self.read_data_from_datastore = 0
            self.tc_datadir.setDisabled(False)
            self.button_d1.setDisabled(False)
            self.tc_format.setDisabled(False)
            self.ntc_every_n.setDisabled(True)
            self.tc_fskey.setDisabled(True)

    # ----------------------------------------------------------------------
    def OnStart(self, evt):

        self.console_info.append("Started DPC batch...")
        QCoreApplication.processEvents()

        QApplication.setOverrideCursor(Qt.WaitCursor)
        self.script_file = "DPCBatchGUIScriptFile.txt"
        self.Save(self.script_file)
        dpc.run_batch(self.script_file)

        QApplication.restoreOverrideCursor()
        self.console_info.append("DPC finished.")

    # ----------------------------------------------------------------------
    def OnSave(self, evt):

        scriptfile = QFileDialog.getSaveFileName(self, "Choose a script file", "", "Text file (*.txt)")[0]

        if scriptfile == "":
            return

        scriptfile = os.path.abspath(scriptfile)

        self.Save(scriptfile)

    # ----------------------------------------------------------------------
    def Save(self, scriptfile):

        # Get the info
        self.scan_range = self.tc_scan_range.text()
        self.settings.setValue("scan_range", self.scan_range)
        # self.scan_nums = self.tc_scans.text()
        # self.settings.setValue('scan_nums', self.scan_nums)
        if self.scan_range == "":
            QMessageBox.warning(self, "Error", "Please enter scan range or scan number.")
            return

        self.every_n = self.ntc_every_n.text()
        self.settings.setValue("every_n", self.every_n)
        if self.cb_usedatastore.isChecked():
            self.read_data_from_datastore = 1
        else:
            self.read_data_from_datastore = 0
        self.settings.setValue("read_data_from_datastore", self.read_data_from_datastore)
        self.filestore_key = self.tc_fskey.text()
        self.settings.setValue("filestore_key", self.filestore_key)
        self.data_directory = self.tc_datadir.text()
        self.settings.setValue("data_dir", self.data_directory)
        if (self.data_directory == "") and (self.read_data_from_datastore == 0):
            QMessageBox.warning(self, "Error", "Please enter data directory or read from DataStore.")
            return
        self.file_format = self.tc_format.text()
        self.settings.setValue("file_format", self.file_format)
        if self.cb_paramsdatastore.isChecked():
            self.load_params_from_broker = 1
        else:
            self.load_params_from_broker = 0
        self.settings.setValue("load_params_datastore", self.load_params_from_broker)
        self.parameter_file = self.tc_paramfile.text()
        self.settings.setValue("param_file", self.parameter_file)
        if self.parameter_file == "":
            QMessageBox.warning(self, "Error", "Please enter scan parameter file.")
            return
        self.processes = self.ntc_processes.text()
        self.settings.setValue("processes", self.processes)
        self.save_dir = self.tc_savedir.text()
        if self.save_dir == "":
            QMessageBox.warning(self, "Error", "Please enter save directory.")
            return
        self.settings.setValue("save_dir", self.save_dir)
        self.save_filename = self.tc_savefn.text()
        if self.save_filename == "":
            QMessageBox.warning(self, "Error", "Please enter save file name.")
            return
        self.settings.setValue("save_fn", self.save_filename)
        if self.cb_savepng.isChecked():
            self.save_png = 1
        else:
            self.save_png = 0
        self.settings.setValue("save_png", self.save_png)
        if self.cb_savetxt.isChecked():
            self.save_txt = 1
        else:
            self.save_txt = 0
        self.settings.setValue("save_txt", self.save_txt)

        # Save the info into script file
        self.console_info.append("\n#DPC script file")
        if self.scan_range != "":
            self.console_info.append("scan_range = {0}".format(self.scan_range))
        # if self.scan_nums != '':
        #     self.console_info.append('scan_numbers = {0}'.format(self.scan_nums))
        self.console_info.append("every_nth_scan = {0}".format(self.every_n))
        self.console_info.append("get_data_from_datastore = {0}".format(self.read_data_from_datastore))
        self.console_info.append("file_store_key = {0}".format(self.filestore_key))
        self.console_info.append("data_directory = {0}".format(self.data_directory))
        self.console_info.append("file_format = {0}".format(self.file_format))
        self.console_info.append("parameter_file = {0}".format(self.parameter_file))
        self.console_info.append("read_params_from_datastore = {0}".format(self.load_params_from_broker))
        self.console_info.append("processes = {0}".format(self.processes))
        self.console_info.append("save_path = {0}".format(self.save_dir))
        self.console_info.append("save_filename = {0}".format(self.save_filename))
        self.console_info.append("save_pngs = {0}".format(self.save_png))
        self.console_info.append("save_txt = {0}".format(self.save_txt))

        try:
            sf = open(scriptfile, "w")
            sf.write("#DPC script file\n")
            if self.scan_range != "":
                sf.write("scan_range = {0}\n".format(self.scan_range))
            # if self.scan_nums != '':
            #     sf.write('scan_numbers = {0}\n'.format(self.scan_nums))
            sf.write("every_nth_scan = {0}\n".format(self.every_n))
            sf.write("get_data_from_datastore = {0}\n".format(self.read_data_from_datastore))
            sf.write("file_store_key = {0}\n".format(self.filestore_key))
            sf.write("data_directory = {0}\n".format(self.data_directory))
            sf.write("file_format = {0}\n".format(self.file_format))
            sf.write("parameter_file = {0}\n".format(self.parameter_file))
            sf.write("read_params_from_datastore = {0}\n".format(self.load_params_from_broker))
            sf.write("processes = {0}\n".format(self.processes))
            sf.write("save_path = {0}\n".format(self.save_dir))
            sf.write("save_filename = {0}\n".format(self.save_filename))
            sf.write("save_pngs = {0}\n".format(self.save_png))
            sf.write("save_txt = {0}\n".format(self.save_txt))
            sf.close()

            self.console_info.append("\nSaved script file {0}".format(scriptfile))
        except Exception:
            QMessageBox.warning(self, "Error", "Error writing script file!")
            return

        self.script_file = scriptfile


""" ------------------------------------------------------------------------------------------------"""


def main():

    app = QApplication(sys.argv)
    frame = MainFrame()
    frame.show()

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



"""
SPEC file format writer
... and partial reader, but it's recommended to use Specfile instead
"""
import re
import os
import time
import itertools
import numpy as np

TIME_FORMAT = "%a %b %d %H:%M:%S %Y"


def split_sequence(iterable, size):
    """
    Take an iterable sequence (not just lists), and split it into
    equal sized chunks (<= size).
    """
    it = iter(iterable)
    item = list(itertools.islice(it, size))
    while item:
        yield item
        item = list(itertools.islice(it, size))


class SPECFileError(Exception):
    pass


class SPECFileMotorListError(SPECFileError):
    pass


class SPECFileWriter(object):
    """
    Writes SPEC-format files for a scan.
    Simple example:

        motors = ['m0', 'm1']
        data_names = ['test', 'one', 'two']
        a = SPECFileWriter('test.txt', comment='my comment', motors=motors)
        a.write_scan_start(command='dscan something', seconds=1)
        a.write_motor_positions([3, 4])
        a.write_scan_data_start(data_names)
        for d1, d2, d3 in zip(range(10), range(2,12), range(10,20)):
            a.write_scan_data([d1,d2,d3])
        a.finish_scan()

    Remember that SPEC scan counts are 0-based -- this means that 0 data points
    is actually 1 data point in EPICS.
    """

    COLUMNS = 8

    def __init__(self, filename, starting_number=None, comment="", motors=[]):
        if not filename:
            raise SPECFileError("Must specify filename")

        self.filename = os.path.abspath(filename)
        self._motors = list(sorted(motors))
        self._comment = comment

        self._data_lines = 0  # how many lines in the current scan there are
        # Scan headers are not written until there's at least one data point
        # available. (otherwise, the C-based spec file reader can fail)
        self._buffer_write = False
        self._buffer = []

        if os.path.exists(filename) and os.path.getsize(filename) > 1:
            if not check_motor_list(filename, motors):
                raise SPECFileMotorListError("Motor list does not match")

            with SPECFileReader(filename) as reader:
                self.start_time = reader.epoch()

                if starting_number is None:
                    self.scan_number = get_last_scan_number(reader)
                else:
                    self.scan_number = starting_number

            print('SPECFileWriter: Appending to "%s"' % filename)
            self._f = open(filename, "at")
            self._fix_ending_newlines(filename)

            self._header_written = True
        else:
            print('SPECFileWriter: New SPEC-format file "%s"' % filename)
            if starting_number is None:
                starting_number = 0

            self.scan_number = starting_number
            self._f = open(filename, "wt")
            self._header_written = False

    def _get_motors(self):
        return list(self._motors)

    def _set_motors(self, motors):
        if self._header_written:
            if motors != self._motors:
                raise SPECFileMotorListError("Cannot change motor list when header is already written")
            return

        self._motors = list(motors)

    def _fix_ending_newlines(self, filename=None, amount=2):
        if filename is None:
            filename = self.filename

        count = self._check_end_lines(filename, amount)
        for i in range(amount - count):
            self.blank_line()

    def _check_end_lines(self, filename, ending=2, char="\n"):
        try:
            f = open(filename, "rt")
            # Seek two bytes before the end of the file
            f.seek(-ending, 2)
            return f.read().count(char)
        except Exception:
            return 0

    def close(self):
        if self._f:
            self._f.close()
            self._f = None

    def write_line(self, line):
        line = "%s\n" % line
        if self._buffer_write:
            self._buffer.append(line)
        else:
            if self._buffer:
                self._f.writelines(self._buffer)
                self._buffer = []

            self._f.write(line)

    def write_info(self, tag, data):
        self.write_line("#%s %s" % (tag, data))

    def write_scan_start(self, number=None, command="", scan_info=None, seconds=None):
        """
        Writes the following:
        Scan information, date, wait time

        In the format:
        #S 1  hklscan  0.9 1.1  0 0  0 0  20 1
        #D Wed Feb 17 19:25:55 1994
        #T 1  (Seconds)
        """
        if not self._header_written:
            self._write_header(self._comment, self._motors)

        if self._data_lines > 0:
            self.finish_scan()

        if number is None:
            self.scan_number += 1
            number = self.scan_number

        # Buffer this scan header until data comes in
        self._buffer_write = True

        # S - scan number and command
        self.write_info("S", "%d  %s" % (number, command))
        self.write_date()
        if seconds is not None:
            # T - scan/settling time
            self.write_info("T", "%f (Seconds)" % (seconds,))

    def write_scan_data_start(self, columns):
        if columns is None:
            columns = []

        self._data_lines = 0

        # N - Number of columns
        self.write_info("N", len(columns))
        # L - Column names (double-space separated)
        self.write_info("L", "  ".join(columns))

    def write_motor_positions(self, positions):
        if not positions:
            return

        for i, pos in enumerate(split_sequence(positions, self.COLUMNS)):
            # P{num} - starting motor positions (that aren't necessarily being
            # scanned)
            tag = "P%d" % i
            self.write_info(tag, " ".join([str(p) for p in pos]))

    def write_scan_data(self, data):
        self._buffer_write = False
        self._data_lines += 1
        self.write_line(" ".join([str(d) for d in data]))

    def write_mca_calib(self, a, b, c):
        """
        Ref: http://www.esrf.eu/blissdb/macros/macdoc.py?macname=saveload.mac

        CALIBRATION = a + b*CHANNEL + c*CHANNEL^2
        """
        self.write_line("#@CALIB  %.7g %.7g %.7g" % (a, b, c))

    def write_mca_data(self, data, calibration=None, first=False):
        """ """
        if data is None or np.size(data) == 0:
            return

        self._buffer_write = False
        self._data_lines += 1
        # does it really support multiple sets of MCA data per scan?
        # if isinstance(data, (list, tuple)):
        #    if isinstance(data[0], np.ndarray):
        #        for array_ in data:
        #            self.write_mca_data(array_, calibration=calibration, first=first)
        #        return

        if first:
            # MCA format (full line)
            self.write_line("#@MCA %%%dC" % len(data))

            # number of channels, first idx, last idx, reduction coefficient
            self.write_line("#@CHANN  %d 0 %d 1" % (len(data), len(data) - 1))

            if calibration:
                self.write_mca_calib(*calibration)

        self.write_line("@A %s" % " ".join([str(d) for d in data]))

    @property
    def date_string(self):
        # return time.strftime('%c')
        # TODO maybe this ^^ is more appropriate for other locales?
        return time.strftime(TIME_FORMAT)

    def _write_starting_seconds(self):
        self.start_time = time.time()
        self.write_info("E", int(self.start_time))

    def write_date(self):
        self.write_info("D", self.date_string)

    def write_timestamp(self):
        self.write_info("E", self.date_string)

    def write_scan_aborted(self, points_written=None):
        self._buffer_write = False

        if points_written is not None:
            self._data_lines = points_written

        if self._data_lines > 0:
            self.write_info("C", "%s.  Scan aborted after %d points." % (self.date_string, self._data_lines))
            self.blank_line()
            self._data_lines = 0
            self._f.flush()
        else:
            # Cancelled before it even started. Don't even write the
            # buffered header.
            self._buffer = []

    def blank_line(self):
        self.write_line("")

    def _write_header(self, comment="", motors=[]):
        self._header_written = True
        self.write_info("F", self.filename)
        self._write_starting_seconds()
        self.write_date()
        if comment is not None:
            for line in comment.split("\n"):
                self.write_info("C", line)  # TODO multiple lines valid?

        for i, mot in enumerate(split_sequence(motors, self.COLUMNS)):
            # O{num} - motor names (that aren't necessarily being scanned)
            tag = "O%d" % i
            self.write_info(tag, "  ".join([str(m) for m in mot]))

        self.blank_line()
        self._data_lines = 0

    def finish_scan(self):
        self._fix_ending_newlines()

        self._data_lines = 0
        self._f.flush()


class SPECFileReader(object):
    """
    NOTE: Really untested except for reading the header.
    (just needed a safe, quick way to check the motor list.)
    """

    def __init__(self, filename, parse_data=True):
        if not os.path.exists(filename):
            raise ValueError("Invalid SPEC filename")

        self._f = open(filename, "rt")
        self._in_scan = False
        self._buffer_lines = []
        self._buffer_groups = []
        self._scans = []
        self._scan = None
        self._eof = False
        self._epoch = time.time()
        self._mca_line = False
        self._parse_data = parse_data

        self.spec_filename = filename
        self.motors = []
        self.comment = ""

        self._read_header()

    def close(self):
        if self._f:
            self._f.close()
            self._f = None

    def scanno(self):
        return 0  # TODO

    def epoch(self):
        return self._epoch

    def _read_line(self):
        while True:
            while self._buffer_lines:
                line = self._buffer_lines.pop(0)
                yield line

            line = self._f.readline()
            if line == "":
                yield "#S"  # TODO: fix so this isn't necessary
                self._eof = True
                break

            line = line.strip()
            if line and (line.startswith("#") or self._in_scan):
                yield line

    def _read_group(self):
        while self._buffer_groups:
            yield self._buffer_groups.pop(0)

        current_tag = None
        group = []
        tag_done = False

        for line in self._read_line():
            if self._eof:
                if current_tag is not None:
                    yield current_tag.upper(), group

                break

            if line.startswith("#"):
                # print('-> %s' % line)
                if " " in line[1:]:
                    tag, info = line[1:].split(" ", 1)
                else:
                    tag, info = line[1], ""

                info = info.lstrip()

                m = re.match(r"([@a-zA-Z]+)(\d*)", tag)
                if m:
                    tag, tag_index = m.groups()
                else:
                    tag_index = None

                # print('current', current_tag, 'read tag', tag, 'index',
                # tag_index)
                if current_tag is None:
                    current_tag = tag
                    group.append(info)
                    if tag_index is None:
                        tag_done = True
                else:
                    if tag == current_tag:
                        group.append(info)
                    else:
                        self._buffer_lines.append(line)
                        tag_done = True

            elif self._in_scan:
                if self._parse_data:
                    self._parse_scan_line(line)
                else:
                    self._scan["unparsed"].append(line)

            if tag_done:
                yield current_tag.upper(), group

                current_tag = None
                group = []
                tag_done = False

    def _parse_list(self, list_):
        list_ = re.sub(r"\s+", ",", list_)
        return list_.split(",")

    def _parse_header_F(self, spec_filename):
        self.spec_filename = spec_filename

    def _parse_header_D(self, date_):
        self._epoch = time.mktime(time.strptime(date_, TIME_FORMAT))

    def _parse_header_list_O(self, motors):
        self.motors = motors

    def _parse_header_C(self, comment):
        self.comment = comment

    def _read_section(self, section, end_tags=None, ignore_first_tag=False):
        first_tag = True
        for tag, lines in self._read_group():
            if self._eof:
                return

            if end_tags is not None and tag in end_tags or self._eof:
                if not (first_tag and ignore_first_tag):
                    self._buffer_groups.insert(0, (tag, lines))
                    break

            # print('section', section, tag, lines)
            lines = "  ".join(lines)
            fcn_name = "_parse_%s_list_%s" % (section, tag)
            if hasattr(self, fcn_name):
                fcn = getattr(self, fcn_name)
                fcn(self._parse_list(lines))

            fcn_name = "_parse_%s_%s" % (section, tag)
            if hasattr(self, fcn_name):
                # print('calling', fcn_name)
                fcn = getattr(self, fcn_name)
                fcn(lines)

            first_tag = False

    def _read_header(self):
        self._read_section("header", end_tags=["S"])

    def _parse_scan_S(self, scan_info):
        number, command = scan_info.split(" ", 1)

        self._in_scan = True
        self._scan = {
            "lines": [],
            "mca_data": [],
            "columns": [],
            "time": 0,
            "hkl": [0, 0, 0],
            "fourc": [],
            "positions": [],
        }

        if not self._parse_data:
            self._scan["unparsed"] = []

        self._scan["number"] = number
        self._scan["command"] = command.strip()

        self._scans.append(self._scan)

    def _parse_scan_list_L(self, columns):
        self._scan["columns"] = columns

    def _parse_scan_D(self, date_):
        self._scan["time"] = time.mktime(time.strptime(date_, TIME_FORMAT))

    def _parse_scan_list_G(self, fourc_info):
        self._scan["fourc"] = fourc_info

    def _parse_scan_list_Q(self, hkl):
        self._scan["hkl"] = hkl

    def _parse_scan_line(self, line):
        if line.startswith("@A"):
            line = line.split(" ", 1)[1]

            # Line is now everything after @A
            data = [float(f) for f in line.split(" ")]

            # If the MCA data is spread on multiple lines, extend the previous
            # data, otherwise it's a new set
            if self._mca_line:
                self._scan["mca_data"][-1].extend(data)
            else:
                self._scan["mca_data"].append(data)

            self._mca_line = line.endswith("\\")
        else:
            try:
                line = line.replace("None", "0.0")  # TODO during saving
                self._scan["lines"].append([float(f) for f in line.split(" ")])
            except Exception:
                print("Bad scan line: %s" % line)

    def parse_data(self, scan):
        """
        Parse the scan data after the file is loaded, if parse_data was set
        """
        if self._parse_data:
            return

        if "unparsed" not in scan:
            return

        self._scan = scan
        for line in scan["unparsed"]:
            self._parse_scan_line(line)

        del scan["unparsed"]

    def _parse_scan_list_P(self, positions):
        positions = [float(p) if p != "None" else 0.0 for p in positions]  # TODO fix
        self._scan["positions"] = dict(zip(self.motors, positions))

    @property
    def scans(self):
        while True:
            scan = self.read_scan()
            if scan is None:
                break

            yield scan

    def read_scan(self):
        if self._eof:
            return None

        self._read_section(
            "scan",
            end_tags=[
                "S",
            ],
            ignore_first_tag=True,
        )
        # should 'C' really be an end_tag, since you can use scan_on?
        # what does scan_on output look like?
        self._in_scan = False

        return self._scan

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()


def check_motor_list(filename, motors):
    try:
        sf = SPECFileReader(filename)
    except Exception:
        return False
    return motors == sf.motors


def get_last_scan_number(filename):
    numbers = [0]
    try:
        if isinstance(filename, SPECFileReader):
            reader = filename
        else:
            reader = SPECFileReader(filename)

        for scan in reader.scans:
            try:
                numbers.append(int(scan["number"]))
            except Exception:
                pass

        return max(numbers)

    except Exception as ex:
        print("Failed to get last scan number: (%s) %s" % (filename, ex))
        return 0


if __name__ == "__main__":
    motors = ["m0", "m1"]
    data_names = ["test", "one", "two"]
    a = SPECFileWriter("test.txt", comment="my comment", motors=motors)
    a.write_scan_start(command="dscan something", seconds=1)
    a.write_motor_positions([3, 4])
    a.write_scan_data_start(data_names)
    for d1, d2, d3 in zip(range(10), range(2, 12), range(10, 20)):
        a.write_scan_data([d1, d2, d3])
    a.finish_scan()

    # reader = SPECFileReader('/epics/data/aug_21_11')
    reader = SPECFileReader("../test_output")
    for scan in reader.scans:
        print(scan["number"], scan["command"], scan.keys(), scan["columns"])

    print(reader._scans[-1], len(reader._scans))
    print(reader._buffer_groups)
    print(reader._scans[-1]["lines"])
    print(reader._scans[-1]["columns"])
    print(reader._scan["columns"])
